<?php

/*sisällytä config- tiedosto
The require_once() function in PHP
is used to include one PHP file into another PHP file.
*/
require_once "config.php";

/*määrittele muuttujat ja alusta tyhjillä arvoilla
_err tarkoittaa ilmestyvää error- viestiä, mikäli syöte ei vastaa annettuja ehtoja*/
$username = $password = $confirm_password = "";
$username_err = $password_err = $confirm_password_err = "";

//Käsittelee lomakkeen tietoja, kun lomake lähetetään
if($_SERVER["REQUEST_METHOD"] == "POST")
{
    
    /*Todennetaan käyttäjänimi
     Tarkista onko käyttäjänimi-kenttä  tyhjä*/
    if(empty(trim($_POST["username"]))){
       $username_err = "Anna käyttäjänimi.";
       
        /*preg_match — Perform a regular expression match
        Searches subject for a match to the regular expression given in pattern.
        etsii määriteltyjä merkkejä ja jos eri kuin (!pregmatch) niin ilmoittaa*/
    }elseif(!preg_match('/^[a-zA-Z0-9_]+$/', trim($_POST["username"]))){
         $username_err = "Käyttäjänimi voi sisältää vain kirjaimia,numeroita ja alaviivoja.";
}else
{
    /*jos käytössä sallitut merkit, antaa edetä
    
    Valmistele SELECT-lause
    annetulle usernamelle määräytyy auto_increment (järjestyksessä)
    id- luku
    seuraava lause valitsee annettua username:a vastaavan id:n arvon*/
    $sql = "SELECT id FROM users WHERE username = ?";
    
    //mysqli_prepare — Prepares an SQL statement for execution
    if($stmt = mysqli_prepare($con, $sql))
    {
        
        /*(bind_param)
            When you prepare an SQL statement, 
            you can insert a placeholder where a column value would go,
            then use bind_param() to safely substitute
            that placeholder for the real column's value. 
            This prevents any possibility of an SQL injection.
        sido muuttujat valmisteltuun komentoon parametreina  */
        mysqli_stmt_bind_param($stmt, "s", $param_username);
        
        //aseta parametrit
        $param_username = trim($_POST["username"]);
        
        /*Kokeile suorittaa valmisteltu komento
        mysqli_stmt_execute — Executes a prepared statement*/
        if(mysqli_stmt_execute($stmt)){
            //tallenna tulos
            mysqli_stmt_store_result($stmt);
            
              //Tarkista onko käyttäjänimi olemassa
            if(mysqli_stmt_num_rows($stmt) == 1){
                $username_err = "Tämä käyttäjätunnus on jo käytössä.";
            }else{
                 $username = trim($_POST["username"]);
            }
            
        }else{
            echo "Voi ei! Jotain meni pieleen. Yritä uudelleen.";
        }
        //Sulje komento
        mysqli_stmt_close($stmt);
        }
    }
    //Todenna salasana
    if(empty(trim($_POST["password"]))){
        $password_err = "Anna salasana";
    }elseif(strlen(trim($_POST["password"])) <6){
        $password_err = "Salasanan on oltava vähintään 6 merkkiä.";
    }else{
        $password = trim($_POST["password"]);
    }
    
    //Vahvista salasana
    if(empty(trim($_POST["confirm_password"]))){
        $confirm_password_err = "Vahvista salasana.";
    }else{
         $confirm_password = trim($_POST["confirm_password"]);
         if(empty($username_err) && ($password != $confirm_password)){
             $confirm_password_err = "Salasana ei täsmää.";
         }
    }
    
    //Tarkista input-virheet ennen tietokantaan syöttämistä
    if(empty($username_err) && empty($password_err) && empty($confirm_password_err))
    {
        
        //Valmista INSERT- lauseke
        $sql= "INSERT INTO users (username, password) VALUES (?, ?)";
        
        if($stmt = mysqli_prepare($con, $sql)){
            //Sido muuttujat valmisteltuun komentoon parametreinä
            mysqli_stmt_bind_param($stmt, "ss", $param_username, $param_password);
            
            //Aseta parametrit
            $param_username = $username; 
            $param_password = password_hash($password, PASSWORD_DEFAULT);
            //password_hash — Creates a password hash, Luo hash-salasanan 
            
            //Yritä suorittaa valmisteltu komento
            if(mysqli_stmt_execute($stmt)){
                //Suuntaa kirjautumis-sivulle
                header("location: login.php");
            }else{
                echo "Voi ei! Jotain meni pieleen. Kokeile uudestaan.";
            }
            //Sulje komento
            mysqli_stmt_close($stmt);
        } 
    }
    //Sulje yhteys
    mysqli_close($con);
}    
    
?>
<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="UTF-8">
        <title>Sign Up</title>
        <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.5.2/css/bootstrap.min.css">
        <style>
        body{font: 14px sans-serif;}
        .wrapper{width: 360px; padding: 20px;}
        </style>
        </head>
        <body>
            <div class="wrapper">
                <h2>Rekisteröidy</h2>
                <p>Täytä lomake käyttäjätilin luomiseksi. </p>
                <form action="<?php echo htmlspecialchars($_SERVER["PHP_SELF"]); ?>" method="post">
                    <div class="form-group">
                        <label>Käyttäjätunnus</label>
                        <input type="text" name="username" class="form-control <?php echo (!empty($username_err)) ? 'is-invalid' : ''; ?>" value="<?php echo $username; ?>">
                        <span class = "invalid-feedback"><?php echo $username_err; ?></span>
                        </div>
                        
                        <div class="form-group">
                            <label>Salasana</label>
                            <input type="password" name="password" class="form-control <?php echo (!empty($password_err)) ? 'is-invalid' : ''; ?>" value="<?php echo $password; ?>">
                            <span class="invalid-feedback"><?php echo $password_err ?></span>
                            </div>
                            <div class="form-group">
                                <label>Vahvista salasana</label>
                                <input type="password" name="confirm_password" class="form-control <?php echo (!empty($confirm_password_err)) ? 'is-invalid' : ''; ?>" value="<?php echo $confirm_password; ?>">
                                <span class="invalid-feedback"><?php echo $confirm_password_err; ?></span>
                                </div>
                                <div class="form-group">
                                    <input type="submit" class="btn btn-primary" value="Submit">
                                    <input type="reset" class="btn btn-secondary ml-2" value="Reset">
                                    </div>
                                    <p>Onko käyttäjätili jo olemassa?<a href="login.php">Kirjaudu tästä</a>.</p>
                                    </form>
                                    </div>
                                    </body>
                                    </html>
                        
        
        