<?php
/*Aloita sessio. Initialize
session_start() creates a session or resumes the current one based on a session
identifier passed via a GET or POST request*/
session_start();

/*Tarkista ensin onko käyttäjä jo kirjautuneena sisään, 
jos vastaus kyllä, käyttäjä suunnataan suoraan aloitussivulle*/

/*The isset() function is an inbuilt function
in PHP which is used to determine 
if the variable is declared and 
its value is not equal to NULL.
header() is used to send a raw HTTP header
*/
if(isset($_SESSION["loggedin"]) && $_SESSION["loggedin"]===true){
    header("location: welcome.php");
    exit;
}
/*sisällytä config- tiedosto
The require_once() function in PHP
is used to include one PHP file into another PHP file*/
require_once "config.php";

/*määritä muuttujat ja aloita istunto tyhjillä arvoilla
_err tarkoittaa ilmestyvää error -viestiä, kun annettu syöte ei vastaa ehtoja*/
$username = $password = "";
$username_err = $password_err = $login_err = "";

// lomakkeen tiedon käsittely kun lomake lähetetään
if($_SERVER["REQUEST_METHOD"] == "POST"){
    
    /*Haetaan kirjaumistiedot users- tietokannasta todentamista varten
    
    Todennetaan käyttäjänimi
    Tarkista onko käyttäjänimi-kenttä  tyhjä*/
    if(empty(trim($_POST["username"]))){
        $username_err = "Anna käyttäjätunnus.";
    } else{
        $username = trim($_POST["username"]);
    }
    
    /*Tarkista onko salasana- kenttä tyhjä
empty — Determine whether a variable is empty
The trim () function removes the whitespaces 
or other characters from the beginning and end of a string*/ 
    if(empty(trim($_POST["password"]))){
        $password_err = "Anna salasana.";
    } else{
        $password = trim($_POST["password"]);
    }
    
    //Todenna kirjautumistiedot
    if(empty($username_err) && empty($password_err)){
        
        /*Valmista SELECT- lause
        hakee annettua username:a vastaavat id, username ja password- tiedot*/
        $sql = "SELECT id, username, password FROM users WHERE username = ?";
        
        //mysqli_prepare — Prepares an SQL statement for execution
        if($stmt = mysqli_prepare($con, $sql)){
            
            /*(bind_param)
            When you prepare an SQL statement, 
            you can insert a placeholder where a column value would go,
            then use bind_param() to safely substitute
            that placeholder for the real column's value. 
            This prevents any possibility of an SQL injection.
       
            Sitoo muuttujat valmisteltuun komentoon     */
            mysqli_stmt_bind_param($stmt, "s", $param_username);
            
            //Aseta parametrit
            $param_username = $username;
         
            /*Yritetään suorittaa (execute) valmisteltua komentoa
            mysqli_stmt_execute — Executes a prepared statement*/
            if(mysqli_stmt_execute($stmt)){
                //mikäli onnistuu, tallenna tulos
                mysqli_stmt_store_result($stmt);
                
                /*Tarkista onko käyttäjänimi olemassa
                mikäli käyttäjänimi olemassa todenna salasana
                mysqli_stmt_num_rows — Returns the number of rows fetched from the server*/
                if(mysqli_stmt_num_rows($stmt) == 1){
                    
                /*sido tuloksen muuttujat
                    mysqli_stmt_bind_result() function is used to bind the columns of a result set to variables*/
                    mysqli_stmt_bind_result($stmt, $id, $username, $hashed_password);
                    
                    /*mysqli_stmt_fetch — 
                    Fetch results from a prepared statement into the bound variables
                    password_verify — Verifies that a password matches */
                    if(mysqli_stmt_fetch($stmt)){
                        if(password_verify($password, $hashed_password)){
                        
              
                            //jos salasana on oikein, aloita uusi sessio
                            session_start();
                            
                              //tallenna data session-muuttujaan
                               $_SESSION["loggedin"] = true;
                            $_SESSION["id"] = $id;
                            $_SESSION["username"] = $username;
                            
                         //Tallenna login tiedot sessions- tauluun
                      $stmt = $con->prepare("INSERT INTO sessio (user_id, ip_address, ip_addr) VALUES (?,?,?)");  
 $stmt->bind_param("iss", $user_id, $ipv4, $ip);   
                          
 
  //aseta arvot parametreille
  $user_id= $id;
  //tarkista ja hae ip-osoite, sido parametriksi
             $ip = isset($_SERVER['HTTP_CLIENT_IP']) ? $_SERVER['HTTP_CLIENT_IP'] : isset($_SERVER['HTTP _ X_FORWARDED_FOR']) ? $_SERVER['HTTP _ X_FORWARDED_FOR'] : $_SERVER['REMOTE_ADDR'];
             
            
              $ipv4=ip2long($ip);
            
             //suorita SQL- komento
               $stmt->execute(); 

  
                            //Suuntaa käyttäjä Welcome-sivulle
                            header("location: welcome.php");
                        } else{
                            //Salasana ei kelpaa, näytä error-viesti
                            $login_err = "Väärä käyttäjätunnus tai salasana";
                        }
                    }
                }else{
                    //Käyttäjätunnusta ei löydy, näytä error-viesti
                    $login_err = "Väärä käyttäjätunnus tai salasana";
                }
            }else{
                echo "Voi ei! Jotain meni pieleen. Kokeile uudelleen. ";
            }
    

            //Sulje komento
            mysqli_stmt_close($stmt);
        }
    }
    //Sulje yhteys
    mysqli_close($con);
    }

?>
<!DOCTYPE html>
<html lang = "en">
    <head>
        <meta charset = "UTF-8">
        <title>Login</title>
        <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.5.2/css/bootstrap.min.css">
        <style>
        body{font: 14px sans-serif;}
        .wrapper{width: 360px; padding: 20px;}
        </style>
        </head>
        <body>
            <div class ="wrapper">
                <h2>Login</h2>
                <p>Anna kirjautumistiedot</p>
                
                <?php
                if(!empty($login_err)){
                    echo '<div class="alert alert-danger">' . $login_err . '</div>';
                }
                ?>
                <form action ="<?php echo htmlspecialchars($_SERVER["PHP_SELF"]);?>" method="post">
                    <div class="form-group">
                        <label>Käyttäjätunnus</label>
                        <input type="text" name="username" class="form-control <?php echo (!empty($username_err)) ? 'is-invalid' : ''; ?>" value="<?php echo $username; ?>">
                        <span class="invalid-feedback"><?php echo $username_err; ?></span>
                        </div>
                        
                        <div class="form-group">
                            <label>Salasana</label>
                            <input type="password" name="password" class="form-control <?php echo (!empty($password_err)) ? 'is-invalid' : ''; ?>">
                            <span class="invalid-feedback"><?php echo $password_err; ?></span>
                            </div>
                            <div class="form-group">
                                <input type="submit" class="btn btn-primary" value="Login">
                                </div>
                                <p>Puuttuuko käyttäjätili?<a href="register.php"> Rekisteröidy nyt!</a>.</p>
                                </form>
                                </div>
                                </body>
                                </html>
                        
